/*
This library is free software; you can redistribute or/and modify it
under the terms of the lGPL license.
*/
#ifndef _EL_H_
#define _EL_H_

#include <SDL/SDL.h>

/*Surface losing*/
class EL_Lose {
private:
  int num_surfaces,num_real,step;
  SDL_Surface **surfaces;
  SDL_RWops *ops;
void ResizeArray(int n);
public:
EL_Lose(FILE *f,int n_surfaces_,int step_);
int RegisterSurface(SDL_Surface *s);
int RegisterSurfaces(SDL_Surface **s,int n);
int UnregisterSurface(SDL_Surface *s);
int UnregisterSurfaces(SDL_Surface **s,int n);
int StoreSurfaces();
int FreeSurfaces();
int LoadSurfaces();
~EL_Lose();
};

/*String collecting functions*/
typedef struct EL_STRING_Data EL_STRING_Data;
struct EL_STRING_Data {
  int no;
  char *str;
  EL_STRING_Data *next;
  void *misc;
};
EL_STRING_Data *EL_STRING_AddString(EL_STRING_Data *data,char *str,void *misc);
int EL_STRING_DelString_No(EL_STRING_Data **data,int no);
int EL_STRING_DelString(EL_STRING_Data **data,char *str);
EL_STRING_Data *EL_STRING_ByNo(EL_STRING_Data *data,int no);
EL_STRING_Data *EL_STRING_ByString(EL_STRING_Data *data,char *str);
#define EL_STRING_TestNo(data,no) ((EL_STRING_ByNo(data,no)==NULL)?(-1):(0))
#define EL_STRING_TestString(data,str) ((EL_STRING_ByString(data,str)==NULL)?(-1):(0))
int EL_STRING_ChangeByNo(EL_STRING_Data *data,int no,void *misc);
int EL_STRING_ChangeByString(EL_STRING_Data *data,char *str,void *misc);
int EL_STRING_Destroy(EL_STRING_Data *data);

/*Parsing config files*/
typedef int (EL_PARSER_FUNC) (char type,char *name,void *var);
class EL_Parser {
char *name;
unsigned char *buffer;
int buf_size;
EL_STRING_Data *vars;
public:
EL_Parser(char *name);
void AddEntry(char type,char one,char *name,void *var,EL_PARSER_FUNC *loadfunc,EL_PARSER_FUNC *savefunc);
int Load(FILE *LOG);
int Save(FILE *LOG);
~EL_Parser();
};
#define EL_PARSER_UINT8 0
#define EL_PARSER_STRING 1
#define EL_PARSER_INT 2

#define EL_PARSER_DOSET 1
#define EL_PARSER_DORUN 2

#define EL_PARSER_ONCE 0
#define EL_PARSER_MANY 1
#define EL_PARSER_ANY 2

/*Misc functions*/
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
#define EL_SURFACE_COLMASKS 0xff000000,0x00ff0000,0x0000ff00,0
#else
#define EL_SURFACE_COLMASKS 0x000000ff,0x0000ff00,0x00ff0000,0
#endif
SDL_Surface **EL_MISC_LoadTBZImages(char *name);

/* Font functions */
class EL_Font {
SDL_Surface *font;
int w0,h0,linesize;
Uint8 *sym_width,*sym_no;
SDL_Surface *temp;
public:
EL_Font(SDL_Surface *font_,int w0_,int h0_,int linesize_,Uint8 *sym_width_,Uint8 *sym_no_);
void OutText(int x,int y,Uint32 colour,char *c,SDL_Surface *surface);
int TextWidth(char *c);
int TextnWidth(char *s,int n);
int TextHeight(char *c);
int TextnHeight(char *s,int n);
};

/*Sound functions*/

/* AA functions */
typedef void EL_AA_File;
typedef int (EL_AA_Readfunc) (void *info,EL_AA_File *f,void *data,int n);
typedef int (EL_AA_Showfunc) (void *info,EL_AA_Readfunc *func);
int EL_AA_Showvideo(EL_AA_Readfunc *readf,EL_AA_Showfunc *showf,void *info);

/*GUI functions*/
class EL_GUI_Widget {
public:
EL_GUI_Widget(){};
virtual int proceed_click(SDL_MouseButtonEvent *click){return -1;};
virtual void refresh(){};
virtual ~EL_GUI_Widget(){};
};
typedef EL_GUI_Widget *(EL_GUI_WidgetInitFunc) (int x,int y, ...);
int EL_GUI_RegisterWidget(char *theme_name,char *widget_name,EL_GUI_WidgetInitFunc *func);
EL_GUI_WidgetInitFunc *EL_GUI_GetWidget (char *theme_name,char *widget_name);
typedef struct EL_GUI_FormEntry EL_GUI_FormEntry;
struct EL_GUI_FormEntry {EL_GUI_Widget *widget;EL_GUI_FormEntry *next;};
typedef void (EL_GUI_FormHandler) (int event,int info);
class EL_GUI_Form {
public:
SDL_Surface *screen,*back;
int x,y,xs,ys;
int return_value;//-1 - no returning
char form_was_changed;
Uint32 back_colour;
EL_GUI_FormHandler *handler;
EL_GUI_FormEntry *widgets;
EL_GUI_Form(EL_GUI_FormHandler *handler_,int xs_,int ys_,Uint32 back_colour_);
void AddWidget(EL_GUI_Widget *widget);
void Refresh(void);
int Proceed(void);
~EL_GUI_Form();
};
extern EL_GUI_Form *EL_GUI_CurrentForm;

/* GUI string filters */
class EL_GUI_StringFilter {
public:
virtual void create(char *buf){};
virtual int test(char *buf){return 0;};/*0 if string is OK,-1 if not*/
virtual void set(char *buf){};
};
EL_GUI_StringFilter *EL_GUI_StringFilterFunc(int smth);
class EL_GUI_StringFilter_Uint : public EL_GUI_StringFilter {
public:
int min,max,*value;
virtual void create(char *buf);
virtual int test(char *buf);
virtual void set(char *buf);
};
EL_GUI_StringFilter *EL_GUI_StringFilter_UintFunc(int min,int max,int *value);
typedef EL_GUI_StringFilter *(EL_GUI_StringFilterInitFunc) (int smth, ...);
int EL_GUI_RegisterStringFilter(char *filter_name,EL_GUI_StringFilterInitFunc *func);
EL_GUI_StringFilterInitFunc *EL_GUI_GetStringFilter(char *filter_name);
void EL_GUI_RegisterStandartStringFilters(void);

/* GUI widgets */
class EL_GUI_OutwinCloseWidget : public EL_GUI_Widget {
virtual int proceed_click(SDL_MouseButtonEvent *click);
};
EL_GUI_Widget *EL_GUI_OutwinCloseWidgetFunc(int x,int y,void *nothing);
class EL_GUI_TextWidget : public EL_GUI_Widget {
public:
EL_Font *font;
char *text;
Uint32 colour;
int x,y;
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_TextWidgetFunc(int x,int y,EL_Font *font,char *text,Uint32 colour);
class EL_GUI_ButtonWidget : public EL_GUI_Widget {
public:
int x1,y1,x2,y2,event,info;
Uint32 dark,light;
virtual int proceed_click(SDL_MouseButtonEvent *click);
void show(char is_norm);
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_ButtonWidgetFunc(int x1,int y1,int x2,int y2,Uint32 light,Uint32 dark,int event,int info);
class EL_GUI_ImageWidget : public EL_GUI_Widget {
public:
SDL_Surface *image;
SDL_Rect rect;
int x,y;
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_ImageWidgetFunc(int x,int y,SDL_Rect r,SDL_Surface *s);
class EL_GUI_TransparentWidget : public EL_GUI_Widget {
public:
int x,y,xs,ys;
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_TransparentWidgetFunc(int x,int y,int xs,int ys);
class EL_GUI_BorderWidget : public EL_GUI_Widget {
public:
Uint32 colours[2];
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_BorderWidgetFunc(int x,int y,Uint32 col1,Uint32 col2);
class EL_GUI_HeaderWidget : public EL_GUI_Widget {
public:
Uint32 colour;
char size;
int x1,x2;
virtual int proceed_click(SDL_MouseButtonEvent *click);
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_HeaderWidgetFunc(int x1,int x2,Uint32 colour,char size);
class EL_GUI_CloseButtonWidget : public EL_GUI_Widget {
public:
char size;
int x,y,event,info,resault;
virtual int proceed_click(SDL_MouseButtonEvent *click);
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_CloseButtonWidgetFunc(int x,int y,char size,int event,int info,int resault);
class EL_GUI_CloseButtonWidget_Nice : public EL_GUI_CloseButtonWidget {
public:
SDL_Surface *s;
SDL_Rect r;
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_CloseButtonWidget_NiceFunc(int x,int y,char size,int event,int info,int resault,SDL_Rect r,SDL_Surface *s);
class EL_GUI_StringChoiceWidget : public EL_GUI_Widget {
public:
int x,y;
char n,**str,*choice;
EL_Font *font;
Uint32 cols[3];
void show_table(int y0,char ch);
virtual int proceed_click(SDL_MouseButtonEvent *click);
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_StringChoiceWidgetFunc(int x,int y,char n,char **str,EL_Font *font_,Uint32 textcol,Uint32 backcol,Uint32 shcol,char *choice_);
class EL_GUI_StringEditWidget : public EL_GUI_Widget {
public:
int x,y,xs,ticks;
Uint32 cols[2];
char size,*buf;
EL_Font *font;
EL_GUI_StringFilter *filter;
virtual int proceed_click(SDL_MouseButtonEvent *click);
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_StringEditWidgetFunc(int x,int y,int xs,char size,char *buf,Uint32 textcol,Uint32 backcol,int ticks,EL_Font *font,
                                                                                  EL_GUI_StringFilter *filter);
class EL_GUI_SlayerWidget : public EL_GUI_Widget {
public:
int x,y,size,im_size,*n;
char d;
Uint32 cols[2];
virtual int proceed_click(SDL_MouseButtonEvent *click);
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_SlayerWidgetFunc(int x,int y,int size,int im_size,int *n,char d,Uint32 lightcol,Uint32 darkcol);
class EL_GUI_RadioButtonsWidget : public EL_GUI_Widget {
public:
int x,y,no,step,size;
char *curr;
virtual void show_one(int n,char state);
virtual int proceed_click(SDL_MouseButtonEvent *click);
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_RadioButtonsWidgetFunc(int x,int y,int no,char *curr,int step,int size);
class EL_GUI_RadioButtonsWidget_Nice : public EL_GUI_RadioButtonsWidget {
public:
SDL_Rect rects[2];
SDL_Surface *surfaces[2];
virtual void show_one(int n,char state);
};
EL_GUI_Widget *EL_GUI_RadioButtonsWidgetFunc_Nice(int x,int y,int no,char *curr,int step,int size,SDL_Surface *dis_s,SDL_Rect dis_r,
                                            SDL_Surface *en_s,SDL_Rect en_r);
class EL_GUI_CheckButtonsWidget : public EL_GUI_Widget {
public:
int x,y,no,step,size;
char *buttons;
virtual void show_one(int n,char state);
virtual int proceed_click(SDL_MouseButtonEvent *click);
virtual void refresh();
};
EL_GUI_Widget *EL_GUI_CheckButtonsWidgetFunc(int x,int y,int no,char *buttons,int step,int size);
class EL_GUI_CheckButtonsWidget_Nice : public EL_GUI_CheckButtonsWidget {
public:
SDL_Rect rects[2];
SDL_Surface *surfaces[2];
virtual void show_one(int n,char state);
};
EL_GUI_Widget *EL_GUI_CheckButtonsWidgetFunc_Nice(int x,int y,int no,char *buttons,int step,int size,SDL_Surface *dis_s,SDL_Rect dis_r,
                                            SDL_Surface *en_s,SDL_Rect en_r);

void EL_GUI_RegisterStandartWidgets(void);

#endif
